@@ -1,11 +1,17 @@
-# Computational Autonomy for Materials Discovery
 # Copyright Toyota Research Institute 2019
+"""
+Computational Autonomy for Materials Discovery
+
+A python library for simulating and conducting automated
+sequential learning procedures
+
+"""
 import os
 from functools import partial
 from tqdm import tqdm as _tqdm
 
 
-__version__ = "2020.5.19"
+__version__ = "2020.8.28-post1"
 
 CAMD_ROOT = os.path.dirname(os.path.abspath(__file__))
 CAMD_TEST_FILES = os.path.join(CAMD_ROOT, "tests", "test_files")
