@@ -1,4 +1,9 @@
 # Copyright Toyota Research Institute 2019
+"""
+Module for conducting analysis/postprocessing of
+experiments and campaigns.  Contains the Analyzer
+object, which performs this function.
+"""
 
 import abc
 import warnings
@@ -30,7 +35,17 @@ from monty.serialization import loadfn
 
 
 class AnalyzerBase(abc.ABC):
+    """
+    The AnalyzerBase class defines the contract
+    for post-processing experiments and creating
+    a new seed_data object for the agent.
+
+    """
     def __init__(self):
+        """
+        Initialize an Analyzer.  Should contain all necessary
+        state variables for consistent analysis methods
+        """
         self._initial_seed_indices = []
 
     @abc.abstractmethod
@@ -78,6 +93,20 @@ class GenericMaxAnalyzer(AnalyzerBase):
         super(GenericMaxAnalyzer, self).__init__()
 
     def analyze(self, new_experimental_results, seed_data):
+        """
+        Analyzes the results of an experiment by finding
+        the best examples and their scores
+
+        Args:
+            new_experimental_results (pandas.DataFrame): new experimental
+                results to be analyzed
+            seed_data (pandas.DataFrame): past data to include in analysis
+
+        Returns:
+            (pandas.DataFrame): one-row dataframe summarizing past results
+            (pandas.DataFrame): new seed data to be passed to agent
+
+        """
         new_seed = seed_data.append(new_experimental_results)
         self.score.append(np.sum(new_seed["target"] > self.threshold))
         self.best_examples.append(new_seed.loc[new_seed.target.idxmax()])
@@ -104,6 +133,16 @@ class AnalyzeStructures(AnalyzerBase):
     """
 
     def __init__(self, structures=None, hull_distance=None):
+        """
+        Analyzer for structural analysis of jobs
+
+        Args:
+            structures ([Structure]): list of a-priori structures to
+                compare against
+            hull_distance ([float]): hull_distance by which to filter
+                results
+
+        """
         self.structures = structures if structures else []
         self.structure_ids = None
         self.unique_structures = None
@@ -182,8 +221,11 @@ class AnalyzeStructures(AnalyzerBase):
                     elems = set(s.composition.as_dict().keys())
                     if elems == chemsys:
                         self.icsd_structs_inchemsys.append(s)
-                except:
+                # TODO: can we make this exception more specific,
+                #  do we have an example where this fails?
+                except Exception as e:
                     warnings.warn("Unable to process structure {}".format(k))
+                    warnings.warn("Error: {}".format(e))
 
             self.matching_icsd_strs = []
             for i in range(len(structures)):
@@ -220,19 +262,20 @@ class AnalyzeStructures(AnalyzerBase):
         Useful for analysis integrated as part of a campaign itself
 
         Args:
-            jobs:
-            against_icsd:
+            jobs (pd.DataFrame): dataframe of DFT experiment results
+            against_icsd (bool): whether to validate against ICSD or not
 
         Returns:
         """
         self.structure_ids = []
         self.structures = []
         self.energies = []
-        for j, r in jobs.items():
+        for j, r in jobs.iterrows():
             if r["status"] == "SUCCEEDED":
-                self.structures.append(r["result"]["output"]["crystal"])
+                rdict = r['result'].as_dict()
+                self.structures.append(r['result'].final_structure)
                 self.structure_ids.append(j)
-                self.energies.append(r["result"]["output"]["final_energy_per_atom"])
+                self.energies.append(rdict["output"]["final_energy_per_atom"])
         if use_energies:
             return self.analyze(
                 self.structures, self.structure_ids, against_icsd, self.energies
@@ -242,7 +285,11 @@ class AnalyzeStructures(AnalyzerBase):
 
 
 class StabilityAnalyzer(AnalyzerBase):
-    def __init__(self, hull_distance=0.05, parallel=cpu_count(), entire_space=False):
+    """
+    Analyzer object for stability campaigns
+    """
+    def __init__(self, hull_distance=0.05, parallel=cpu_count(), entire_space=False,
+                 plot=True):
         """
         The Stability Analyzer is intended to analyze DFT-result
         data in the context of a global compositional seed in
@@ -258,11 +305,14 @@ class StabilityAnalyzer(AnalyzerBase):
             entire_space (bool): flag for whether to analyze
                 entire space of results or just new chemical
                 space
+            plot (bool): whether to generate plot as part of
+                standard analyze sequence
         """
         self.hull_distance = hull_distance
         self.parallel = parallel
         self.entire_space = entire_space
         self.space = None
+        self.plot = plot
         super(StabilityAnalyzer, self).__init__()
 
     @staticmethod
@@ -308,18 +358,20 @@ class StabilityAnalyzer(AnalyzerBase):
                 seed data according to the experimental results
 
         """
-        # Aggregate seed_data and new experimental results
+        # Check for new results
+        new_comp = new_experimental_results['Composition'].sum()
+        new_experimental_results = new_experimental_results.dropna(subset=['delta_e'])
         new_seed = seed_data.append(new_experimental_results)
+
+        # Aggregate seed_data and new experimental results
         include_columns = ["Composition", "delta_e"]
-        # TODO: resolve this issue with drop_duplicates
         filtered = new_seed[include_columns].drop_duplicates(keep="last").dropna()
 
         if not self.entire_space:
             # Constrains the phase space to that of the target compounds.
             # More efficient when searching in a specified chemistry,
             # less efficient if larger spaces are without specified chemistry.
-            total_comp = new_experimental_results["Composition"].dropna().sum()
-            filtered = filter_dataframe_by_composition(filtered, total_comp)
+            filtered = filter_dataframe_by_composition(filtered, new_comp)
 
         space = self.get_phase_space(filtered)
         new_phases = [p for p in space.phases if p.description in filtered.index]
@@ -343,7 +395,8 @@ class StabilityAnalyzer(AnalyzerBase):
             new_seed.update(new_data)
 
         # Write hull figure to disk
-        self.plot_hull(new_seed, new_experimental_results.index, filename="hull.png")
+        if self.plot:
+            self.plot_hull(filtered, new_experimental_results.index, filename="hull.png")
 
         # Compute summary metrics
         summary = self.get_summary(
@@ -366,7 +419,8 @@ class StabilityAnalyzer(AnalyzerBase):
                 new processed seed
             new_ids ([]): list of index values for those
                 experiments that are "new"
-
+            initial_seed_indices ([]): indices of the initial
+                seed
 
         Returns:
             (DataFrame): dataframe summarizing processed
@@ -406,12 +460,12 @@ class StabilityAnalyzer(AnalyzerBase):
             (pyplot): plotter instance
         """
         # Generate all entries
-        total_comp = df.loc[new_result_ids]["Composition"].dropna().sum()
-        total_comp = Composition(total_comp)
+        total_comp = Composition(df['Composition'].sum())
         if len(total_comp) > 4:
             warnings.warn("Number of elements too high for phase diagram plotting")
             return None
         filtered = filter_dataframe_by_composition(df, total_comp)
+        filtered = filtered.dropna()
 
         # Create computed entry column with un-normalized energies
         filtered["entry"] = [
@@ -581,8 +635,10 @@ class PhaseSpaceAL(PhaseSpace):
         """
         try:
             phase.stability = phase.energy - self.gclp(phase.unit_comp)[0]
-        except:
-            print(phase)
+        # TODO: do we have an example where this fails?  Can we provide
+        #  a more concrete exception?
+        except Exception as e:
+            print(phase, "stability determination failed, error {}".format(e))
             phase.stability = np.nan
         return phase.stability
 
@@ -632,24 +688,20 @@ def update_run_w_structure(folder, hull_distance=0.2, parallel=True):
         if not all([os.path.isfile(fn) for fn in required_files]):
             print("{} ERROR: no seed data, no analysis to be done")
         else:
-            iteration = -1
-            jobs = {}
-            while True:
-                if os.path.isdir(str(iteration)):
-                    jobs.update(
-                        loadfn(os.path.join(str(iteration), "_exp_raw_results.json"))
-                    )
-                    iteration += 1
-                else:
-                    break
             with open("seed_data.pickle", "rb") as f:
                 df = pickle.load(f)
 
             with open("experiment.pickle", "rb") as f:
                 experiment = pickle.load(f)
+                # Hack to update agg_history
+                experiment.update_current_data(None)
+
             all_submitted, all_results = experiment.agg_history
-            st_a = StabilityAnalyzer(hull_distance=hull_distance, parallel=parallel)
-            summary, new_seed = st_a.analyze(df, pd.DataFrame())
+            old_results = df.drop(all_results.index, errors='ignore')
+            new_results = df.drop(old_results.index)
+            st_a = StabilityAnalyzer(
+                hull_distance=hull_distance, parallel=parallel, entire_space=True, plot=False)
+            summary, new_seed = st_a.analyze(new_results, old_results)
 
             # Having calculated stabilities again, we plot the overall hull.
             st_a.plot_hull(
@@ -659,29 +711,32 @@ def update_run_w_structure(folder, hull_distance=0.2, parallel=True):
                 finalize=True,
             )
 
-            stable_discovered = new_seed[new_seed["is_stable"]]
-            s_a = AnalyzeStructures()
-            s_a.analyze_vaspqmpy_jobs(jobs, against_icsd=True, use_energies=True)
-            unique_s_dict = {}
-            for i in range(len(s_a.structures)):
-                if s_a.structure_is_unique[i] and (
-                    s_a.structure_ids[i] in stable_discovered
-                ):
-                    unique_s_dict[s_a.structure_ids[i]] = s_a.structures[i]
+            stable_discovered = new_seed[new_seed["is_stable"].fillna(False)]
 
-            with open("discovered_unique_structures.json", "w") as f:
-                json.dump(dict([(k, s.as_dict()) for k, s in unique_s_dict.items()]), f)
+            # Analyze structures if present in experiment
+            if "structure" in all_results.columns:
+                s_a = AnalyzeStructures()
+                s_a.analyze_vaspqmpy_jobs(all_results, against_icsd=True, use_energies=True)
+                unique_s_dict = {}
+                for i in range(len(s_a.structures)):
+                    if s_a.structure_is_unique[i] and (
+                        s_a.structure_ids[i] in stable_discovered
+                    ):
+                        unique_s_dict[s_a.structure_ids[i]] = s_a.structures[i]
 
-            with open("structure_report.log", "w") as f:
-                f.write("consumed discovery unique_discovery duplicate in_icsd \n")
-                f.write(
-                    str(len(all_submitted))
-                    + " "
-                    + str(len(stable_discovered))
-                    + " "
-                    + str(len(unique_s_dict))
-                    + " "
-                    + str(len(s_a.structures) - sum(s_a._not_duplicate))
-                    + " "
-                    + str(sum([not i for i in s_a._icsd_filter]))
-                )
+                with open("discovered_unique_structures.json", "w") as f:
+                    json.dump(dict([(k, s.as_dict()) for k, s in unique_s_dict.items()]), f)
+
+                with open("structure_report.log", "w") as f:
+                    f.write("consumed discovery unique_discovery duplicate in_icsd \n")
+                    f.write(
+                        str(len(all_submitted))
+                        + " "
+                        + str(len(stable_discovered))
+                        + " "
+                        + str(len(unique_s_dict))
+                        + " "
+                        + str(len(s_a.structures) - sum(s_a._not_duplicate))
+                        + " "
+                        + str(sum([not i for i in s_a._icsd_filter]))
+                    )
